from nonebot.internal.driver import Request

from typing import Optional, Any, Dict, Union
from enum import Enum

import json

class ContentType(Enum):
    """在 VoceChat 文档中的 ContentType"""

    TEXT_PLAIN = "text/plain"
    TEXT_MARKDOWN = "text/markdown"
    VOCECHAT_FILE = "vocechat/file"
    VOCECHAT_AUDIO = "vocechat/audio"
    VOCECHAT_ARCHIVE = "vocechat/archive"

    def __str__(self):
        return self.value

class API:

    @staticmethod
    def bot(public_only: bool):
        request = Request(
            "GET",
            "/api/bot",
            headers = {
                "Accept": "application/json; charset=utf-8"
            },
            params= {"public_only": public_only}
        )

        return request
    
    @staticmethod
    def send_to_user(uid: int, content_type: Union[ContentType, str], content: Any, properties: Optional[Dict[str, Any]] = None):

        if isinstance(content_type, ContentType):
            content_type = content_type.value

        headers = {
            "Accept": "application/json; charset=utf-8",
            "Content-Type" : content_type
        }

        if properties:
            headers["X-Properties"] = json.dumps(properties)

        if isinstance(content, Dict):
            content = json.dumps(content)

        request = Request(
            "POST",
            f"/api/bot/send_to_user/{uid}",
            headers = headers,
            data= content
        )

        return request
    

    @staticmethod
    def send_to_group(gid: int, content_type: Union[ContentType, str], content: Any, properties: Optional[Dict[str, Any]] = None):

        if isinstance(content_type, ContentType):
            content_type = content_type.value

        headers = {
            "Accept": "application/json; charset=utf-8",
            "Content-Type" : content_type
        }

        if properties:
            headers["X-Properties"] = json.dumps(properties)

        if isinstance(content, Dict):
            content = json.dumps(content)

        request = Request(
            "POST",
            f"/api/bot/send_to_group/{gid}",
            headers = headers,
            data= content
        )

        return request
    

    @staticmethod
    def reply(mid: int, content_type: Union[ContentType, str], content: Any, properties: Optional[Dict[str, Any]] = None):

        if isinstance(content_type, ContentType):
            content_type = content_type.value

        headers = {
            "Accept": "application/json; charset=utf-8",
            "Content-Type" : content_type
        }

        if properties:
            headers["X-Properties"] = json.dumps(properties)

        if isinstance(content, Dict):
            content = json.dumps(content)

        request = Request(
            "POST",
            f"/api/bot/reply/{mid}",
            headers = headers,
            data= content
        )

        return request
    
    @staticmethod
    def send_mail(data: Any, content: Any):

        if isinstance(content, Dict):
            content = json.dumps(content)

        request = Request(
            "POST",
            "/api/bot/send_mail",
            headers = {
                "Accept": "*/*",
                "Content-Type": "application/json; charset=utf-8"
            },
            data= data
        )

        return request
    
    @staticmethod
    def user(uid: int):
        request = Request(
            "GET",
            f"/api/bot/user/uid?uid={uid}",
            headers = {
                "Accept": "application/json; charset=utf-8",
            },
        )

        return request
    
    @staticmethod
    def group(gid: int):
        request = Request(
            "GET",
            f"/api/bot/group/gid?gid={gid}",
            headers = {
                "Accept": "application/json; charset=utf-8",
            },
        )

        return request
    
    @staticmethod
    def file_prepare(content_type: Union[str, ContentType], filename: str):

        if isinstance(content_type, ContentType):
            content_type = content_type.value

        request = Request(
            "POST",
            "/api/bot/file/prepare",
            headers = {
                "Accept": "application/json; charset=utf-8",
                "Content-Type": "application/json; charset=utf-8"
            },
            json= {"content_type": content_type, "filename": filename}
        )

        return request
    

    @staticmethod
    def file_upload(file_id: str, chunk_data: bytes, chunk_is_last: bool):
        files = {
            "file_id": (None, file_id),
            "chunk_is_last": (None, str(chunk_is_last).lower()),
            "chunk_data": ("chunk", chunk_data, "application/octet-stream")
        }
        
        request = Request(
            "POST",
            "/api/bot/file/upload",
            headers={
                "Accept": "application/json; charset=utf-8",
            },
            files=files
        )
        
        return request

    @staticmethod
    def command_add(command: str, description: str):
        request = Request(
            "POST",
            "/api/bot/command",
            headers = {
                "Accept": "application/json; charset=utf-8",
            },
            json= {"command": command, "description": description}
        )

        return request

    @staticmethod
    def command_get():
        request = Request(
            "GET",
            "/api/bot/command",
            headers = {
                "Accept": "application/json; charset=utf-8",
            },
        )

        return request

    @staticmethod
    def command_delete(id: int):
        request = Request(
            "DELETE",
            f"/api/bot/command/{id}",
            headers = {
                "Accept": "application/json; charset=utf-8",
            }
        )

        return request

    @staticmethod
    def command_update(id: int, command: str, description: str):
        request = Request(
            "PUT",
            f"/api/bot/command/{id}",
            headers = {
                "Accept": "application/json; charset=utf-8",
            },
            json= {"command": command, "description": description}
        )

        return request
    
    @staticmethod
    def download_file(file_id: str):
        request = Request(
            "GET",
            f"/api/resource/file?file_path={file_id}&download=true",
            headers = {
                "Accept": "application/octet-stream",
            },
        )

        return request


from typing import TYPE_CHECKING, Any, Union, Optional, Dict
from typing_extensions import override

from nonebot.adapters import Bot as BaseBot
from nonebot.drivers import URL, Response
from nonebot.message import handle_event

import mimetypes
import json
import re

from .event import Event, MessageEvent, Target
from .message import Message, MessageSegment, File
from .api import API, ContentType
from .utils import log, get_mime_type

if TYPE_CHECKING:
    from .adapter import Adapter

def _check_at_me(bot: "Bot", event: MessageEvent) -> None:
    """检查是否有 @me 的情况，并移除 @机器人ID 及后面的空格
    
    Args:
        bot: Bot 对象（需包含 user_id 属性）
        event: MessageEvent 对象
    """
    if not event.message:
        return
    
    first_msg_seg = event.message[0]
    if first_msg_seg.type != "text":
        return

    user_id = str(getattr(bot, "user_id", ""))
    if not user_id:
        return

    first_text = first_msg_seg.data["text"].lstrip()
    
    if m := re.search(rf"^@{user_id}\s+", first_text):
        event.to_me = True
        first_msg_seg.data["text"] = first_text[m.end():]

def _check_nickname(bot: "Bot", event: MessageEvent) -> None:
    """检查消息开头是否存在昵称，去除并赋值 `event.to_me`。

    Args:
        bot: Bot 对象
        event: MessageEvent 对象
    """
    if not event.message:
        return
    
    first_msg_seg = event.message[0]
    if first_msg_seg.type != "text":
        return

    nicknames = {re.escape(n) for n in bot.config.nickname}
    if not nicknames:
        return
    
    nickname_regex = "|".join(nicknames)
    first_text = first_msg_seg.data["text"]
    if m := re.search(rf"^({nickname_regex})([\s,，]*|$)", first_text, re.IGNORECASE):
        event.to_me = True
        first_msg_seg.data["text"] = first_text[m.end():]  # 移除昵称部分

class Bot(BaseBot):
    """
    VoceChat 协议 Bot 适配。
    """

    @override
    def __init__(self, adapter: "Adapter", self_id: str, **kwargs: Any):
        super().__init__(adapter, self_id)
        self.adapter: "Adapter" = adapter
        self.self_id: str = self_id

        self.user_id: str = kwargs.get("user_id", "")
        self.server_base: URL = URL(kwargs.get("server_base", "http://localhost:3000"))
        self.api_key: str = kwargs.get("api_key", "")

    async def handle_event(self, event: Event) -> None:
        """处理事件"""
        if isinstance(event, MessageEvent):
            event.original_message = event.get_message()
            _check_at_me(self, event)
            _check_nickname(self, event)

        await handle_event(self, event)

    async def send_message(
        self,
        message: Message,
        *,
        user_id: Optional[int] = None,
        group_id: Optional[int] = None,
        reply: Optional[int] = None,
        **kwargs: Any
    ) -> Any:
        """发送消息到指定会话
        user_id group_id reply 三选一
        
        Args:
            message: 要发送的消息
            user_id: 私聊的用户id
            group_id: 群聊的群id
            reply: 回复的消息ID
            **kwargs: 其他参数
        """
        content_type: Union[str, ContentType] = ContentType.TEXT_PLAIN
        content: Any = None
        properties: Any = None
        message_id = None
        request = None
        message.reduce()

        for message_segment in message:
            if message_segment.type == "file":
                try:
                    file: File = message_segment.data["file"]
                    content_type = ContentType.VOCECHAT_FILE

                    if file.file_id:
                        content = file.file_id
                    else:    
                        file_result = await self.upload_file(file)
                        content = {"path": file_result.get("path")}
                        properties = file_result.get("image_properties")

                except Exception as e:
                    log("ERROR", f"File upload failed: {e}")
                    raise RuntimeError(f"Failed to upload file: {e}") from e
                    
            elif message_segment.type == "markdown":
                content_type = ContentType.TEXT_MARKDOWN
                content = message_segment.data.get("text")

            else:
                content = message_segment.data.get("text")
        
            try:
                if reply:
                    request = API.reply(
                        mid=reply,
                        content_type=content_type,
                        content=content,
                        properties=properties
                    ) 
                elif group_id:
                    request = API.send_to_group(
                        gid=group_id,
                        content_type=content_type,
                        content=content,
                        properties=properties
                    )
                elif user_id:
                    request = API.send_to_user(
                        uid=user_id,
                        content_type=content_type,
                        content=content,
                        properties=properties
                    )
                
                if request:
                    message_id = await self.call_api("send_message", request= request)
            except Exception as e:
                log("ERROR", f"Failed to send message: {e}")
                raise

        return message_id

    @override
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        **kwargs: Any,
    ) -> Any:
        """发送消息
        
        Args:
            event: 事件对象
            message: 要发送的消息 可以是字符串、Message或MessageSegment
            **kwargs: 其他参数
        """
        from_uid = getattr(event, "from_uid", None)
        if not from_uid:
            raise ValueError("Event has no from_uid")
        
        target = getattr(event, "target", None)
        if not target:
            raise ValueError("Event has no target")

        if isinstance(message, str):
            msg = Message(MessageSegment.text(message))
        elif isinstance(message, MessageSegment):
            msg = Message(message)
        else:
            msg = message
        
        return await self.send_message(
            message=msg,
            user_id=from_uid,
            group_id=getattr(target, "gid", None),
            **kwargs
        )

    async def download_file(
        self,
        file_id: Optional[str] = None,
        message: Optional[Union[Message, MessageSegment]] = None
    ) -> bytes:
        """
        下载文件

        Args:
            file_id: 文件ID
            message: 文件信息

        Returns:
            文件数据
        """

        if isinstance(message, Message):
            for message_segment in message:
                if message_segment.type == "file":
                    file_id = message_segment.data.get("file").file_id
                    if file_id:
                        break
        if isinstance(message, MessageSegment):
            if message.type == "file":
                file_id = message.data.get("file").file_id

        if not file_id:
            return b""

        response = await self.call_api("download_file", file_id=file_id)
        return response.content

    async def upload_file(self, file: File) -> Dict[str, Any]:
        """
        上传文件
        
        Args:
            file: File对象
        
        Returns:
            包含文件信息的字典，如
            {
                "path": "string",
                "size": 0,
                "hash": "string",
                "image_properties": {
                    "width": 0,
                    "height": 0
                }
            }
            其中 path 为消息发送所需要的字段
        """
        file_data = await file.get_data()
        mime_type = get_mime_type(file_data)
        extension = mimetypes.guess_extension(mime_type)
        file_name = file.filename

        content_type = ContentType.VOCECHAT_FILE # 未知的文件类型
    
        # 根据MIME类型确定内容类型
        if mime_type:
            if mime_type.startswith('audio/'):
                content_type = mime_type
            if mime_type.startswith('image/'):
                content_type = mime_type
            if mime_type.startswith('video/'):
                content_type = mime_type

        if not file_name:
            file_name = mime_type.split('/')[0]

        if extension and not file_name.lower().endswith(extension.lower()):
            file_name += extension

        # 验证文件数据
        if not file_data:
            raise ValueError("File data cannot be empty")
    
        # 验证文件大小
        max_size = 50 * 1024 * 1024  # 50MB
        if len(file_data) > max_size:
            raise ValueError(f"File too large: {len(file_data)} bytes (max: {max_size} bytes)")
    
        try:
            # 准备文件上传
            log("DEBUG", f"Preparing file upload: {file_name} ({len(file_data)} bytes)")
            
            prepare_result: Response = await self.call_api(
                api="file_prepare",
                content_type=content_type,
                filename=file_name, 
                raw= True
            )
            
            # 解析准备结果
            if prepare_result.status_code != 200:
                raise RuntimeError(f"File prepare failed with status {prepare_result.status_code}")
            
            try:
                # 尝试解析 JSON 响应
                prepare_content = prepare_result.content
                file_id= None

                if isinstance(prepare_content, bytes):
                    prepare_content = prepare_content.decode('utf-8')
            
                # 如果是纯字符串 file_id
                if prepare_content.startswith('"') and prepare_content.endswith('"'):
                    file_id = prepare_content.strip('"')
            
            except Exception as e:
                log("ERROR", f"Failed to parse prepare result: {e}")
                raise RuntimeError(f"Invalid prepare response format: {e}")
            
            if not file_id:
                raise RuntimeError("No file_id returned from prepare request")
            
            log("DEBUG", f"File prepared with ID: {file_id}")
            
            # 上传文件数据
            upload_result: Response = await self.call_api(
                api= "file_upload",
                file_id= file_id,
                chunk_data= file_data,
                chunk_is_last= True,
                raw= True
            )

            if upload_result.status_code != 200:
                raise RuntimeError(f"File upload failed with status {upload_result.status_code}")
            
            # 解析上传结果
            try:
                upload_content = upload_result.content
                if isinstance(upload_content, bytes):
                    upload_content = upload_content.decode('utf-8')
            
                # 返回的是 JSON 格式
                result_data = json.loads(upload_content)
            
                # 确保有 path 字段
                if "path" not in result_data:
                    raise RuntimeError("No file path returned from upload")
            
            except json.JSONDecodeError as e:
                log("ERROR", f"Failed to parse upload result as JSON: {e}")
                result_data = {}
            except Exception as e:
                log("ERROR", f"Failed to parse upload result: {e}")
                raise RuntimeError(f"Invalid upload response format: {e}")
            
            log("DEBUG", f"File uploaded successfully: {file_name} -> {result_data.get('path')}")
            
            return result_data
            
        except Exception as e:
            log("ERROR", f"File upload failed for {file_name}: {e}")
            raise RuntimeError(f"File upload failed: {e}") from e


from nonebot import get_plugin_config
from nonebot.drivers import (
    Request,
    ASGIMixin,
    HTTPClientMixin,
    HTTPServerSetup,
    URL
)
from nonebot.drivers import Response, Driver
from nonebot.internal.adapter import Adapter as BaseAdapter

from typing_extensions import override
from typing import Any, Dict, Optional
from datetime import datetime

import inspect
import json

from .bot import Bot
from .config import Config
from .event import *
from .utils import log
from .api import API

class Adapter(BaseAdapter):
    @override
    def __init__(self, driver: Driver, **kwargs: Any):
        super().__init__(driver, **kwargs)
        self.adapter_config: Config = get_plugin_config(Config)

        self.setup()

    @classmethod
    @override
    def get_name(cls) -> str:
        """适配器名称"""
        return "VoceChat"

    def setup(self) -> None:
        if not isinstance(self.driver, ASGIMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} doesn't support asgi server!"
                f"{self.get_name()} Adapter need a asgi server driver to work."
            )
        if not isinstance(self.driver, HTTPClientMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} does not support http client requests! "
                f"{self.get_name()} Adapter need a HTTPClient Driver to work."
            )
        

        @self.on_ready
        def _():
            for vocechat_bot in self.adapter_config.vocechat_bots:
                self.bot_connect(
                    Bot(
                        adapter= self,
                        self_id= vocechat_bot.name,
                        user_id = vocechat_bot.user_id,
                        api_key= vocechat_bot.api_key,
                        server_base= vocechat_bot.server
                        )
                    )

        self.setup_http_server(
            HTTPServerSetup(
                URL("/vocechat/webhook"),
                "POST",
                f"{self.get_name()} Webhook",
                self._handle_http,
            ),
        )
        self.setup_http_server(
            HTTPServerSetup(
                URL("/vocechat/webhook/"),
                "POST",
                f"{self.get_name()} Webhook Slash",
                self._handle_http,
            ),
        )

        self.setup_http_server(
            HTTPServerSetup(
                URL("/vocechat/webhook"),
                "GET",
                f"{self.get_name()} Webhook",
                self._handle_http,
            ),
        )
        self.setup_http_server(
            HTTPServerSetup(
                URL("/vocechat/webhook/"),
                "GET",
                f"{self.get_name()} Webhook Slash",
                self._handle_http,
            ),
        )

    @override
    async def _call_api(self, bot: Bot, api: str, **data: Any) -> Any:
        """`Adapter` 实际调用 api 的逻辑实现函数，实现该方法以调用 api。

        参数:
            api: API 名称
            data: API 数据
        """

        log("DEBUG", f"call api {api}")
        
        request = None
        raw = data.pop("raw", False)

        if hasattr(API, api):
            api_method = getattr(API, api)
            sign = inspect.signature(api_method)

            for param in sign.parameters.values():
                if param.name == "self":
                    continue

                if param.name not in data:
                    if param.default == inspect.Parameter.empty:
                        log("ERROR", f"Missing required parameter: {param.name} for API {api}")
                        data[param.name] = None
                    else:
                        data[param.name] = param.default
            
            request = api_method(**data)
        else:
            request = data.get("request", None)

        if request:
            request.headers["x-api-key"] = bot.api_key
            request.url = URL(f"{bot.server_base}{request.url}")

            try:
                response = await self.request(request)
                if raw:
                    return response
                return json.loads(response.content)
            
            except Exception as e:
                log("ERROR", f"Error calling API {api}: {e}")
                raise e


    async def _handle_http(self, request: Request) -> Response:
        """处理VoceChat Webhook请求"""
        try:
            bot_name = request.url.query.get("bot")
            
            if not bot_name:
                return Response(status_code=404, content="Not Found")

            if request.method == "GET": # 添加机器人需要验证
                return Response(status_code=200)
            
            bot = self.bots.get(bot_name)

            if not bot:
                return Response(status_code=404, content="Not Found Bot")

            try:
                payload = request.json
            except ValueError:
                return Response(status_code=400, content="Invalid JSON")
            
            event = self._parse_event(payload, bot)

            if event:
                await bot.handle_event(event)

            return Response(status_code=200)
        
        except Exception as e:
            log("ERROR", "Error handling VoceChat webhook", e)
            return Response(status_code=500, content=str(e))

    def _parse_event(self, payload: Dict[str, Any], bot: Bot) -> Optional[Event]:
        """解析VoceChat事件"""
        try:
            timestamp = payload.get("created_at", 0)

            # 基础事件字段
            event_data = {
                "created_at": timestamp,
                "from_uid": payload.get("from_uid", 0),
                "mid": payload.get("mid", 0),
                "target": payload.get("target", {}),
                "self_uid": bot.user_id,  # 注入机器人自身ID

                "time": datetime.fromtimestamp(timestamp / 1000) if timestamp is not None else datetime.now(),
                "message_id": payload.get("mid", None),  # 兼容性字段
            }
            
            # 根据事件类型创建不同的事件对象
            detail = payload.get("detail", {})
            
            if isinstance(detail, dict):
                # 新消息事件
                if detail.get("type") == "normal":
                    event_data["detail"] = detail
                    
                    # 文件消息特殊处理
                    if detail.get("content_type") == "vocechat/file":
                        return FileMessageEvent.model_validate(event_data)
                    
                    return MessageNewEvent.model_validate(event_data)
                
                # 回复消息
                elif detail.get("type") == "reply":
                    event_data["detail"] = detail
                    event_data["reply"] = detail.get("mid")
                    return MessageNewEvent.model_validate(event_data)
            
            elif isinstance(detail, dict) and detail.get("type") == "reaction":
                reaction_detail = detail.get("detail", {})
                
                # 编辑消息
                if reaction_detail.get("type") == "edit":
                    event_data["detail"] = detail
                    return MessageEditEvent.model_validate(event_data)
                
                # 删除消息
                elif reaction_detail.get("type") == "delete":
                    event_data["detail"] = detail
                    return MessageDeleteEvent.model_validate(event_data)
            
            log("WARNING", f"Unknown event type: {payload}")
            return None
            
        except Exception as e:
            log("ERROR", "Error parsing VoceChat event", e)
            return None


from typing import Any, Dict, Optional, Literal
from typing_extensions import override

from nonebot.utils import escape_tag
from nonebot.compat import model_dump
from nonebot.adapters import Event as BaseEvent
from pydantic import BaseModel, Field, model_validator
from datetime import datetime

from .message import Message, MessageSegment
from .api import ContentType

class Event(BaseEvent):
    time: Optional[datetime] = None

    @override
    def get_event_name(self) -> str:
        raise ValueError("Event has no name!")

    @override
    def get_type(self) -> str:
        raise ValueError("Event has no type!")

    @override
    def get_event_description(self) -> str:
        return escape_tag(repr(model_dump(self)))

    @override
    def get_message(self) -> Message:
        raise ValueError("Event has no message!")

    @override
    def get_user_id(self) -> str:
        raise ValueError("Event has no context!")

    @override
    def get_session_id(self) -> str:
        raise ValueError("Event has no context!")

    @override
    def is_tome(self) -> bool:
        return False

class Target(BaseModel):
    gid: Optional[int] = None
    uid: Optional[int] = None

class MessageDetail(BaseModel):
    content: Optional[str] = None
    content_type: ContentType = ContentType.TEXT_PLAIN
    expires_in: Optional[int] = None
    properties: Optional[Dict[str, Any]] = None
    type: Literal["reaction", "normal", "reply"] = "normal"

class ReactionDetail(BaseModel):
    detail: Dict[str, Any]
    mid: int
    type: str

class MessageEvent(Event):
    """消息事件基类"""
    created_at: int
    from_uid: int
    mid: int
    target: Target
    self_uid: str  # 机器人自身用户ID，由适配器注入

    # 兼容性字段
    message_id: Optional[int] = None
    to_me: Optional[bool] = None
    message: Optional[Message] = None
    original_message: Optional[Message] = None
    reply: Optional[int] = None

    @override
    def get_type(self) -> str:
        return "message"
    
    @override
    def get_event_name(self) -> str:
        return "message"
    
    @override
    def get_user_id(self) -> str:
        return str(self.from_uid)
    
    @override
    def get_session_id(self) -> str:
        if self.target.gid:
            return f"gid_{self.target.gid}_uid_{self.from_uid}"
        elif self.target.uid:
            return f"uid_{self.from_uid}"
        return str(self.from_uid)

    @override
    def is_tome(self) -> bool:
        # 私聊消息直接判断目标是否是机器人
        if self.target.uid and self.target.uid == int(self.self_uid):
            return True
        
        # 群聊消息检查是否 @机器人
        if hasattr(self, "detail") and getattr(self.detail, "properties", None):  # type: ignore
            mentions = self.detail.properties.get("mentions", [])  # type: ignore
            if mentions and int(self.self_uid) in mentions:
                return True
        
        return False
    
    @override
    def get_event_description(self) -> str:
        target_type = "Group" if self.target.gid else "Private"
        target_id = self.target.gid or self.target.uid
        if target_type == "Private":
            return escape_tag(
                f"Message {self.mid} from: {self.from_uid}: "
                f"{self.get_message()}"
            )
        
        return escape_tag(
                f"Message {self.mid} from: {self.from_uid}@[群:{target_id}]: "
                f"{self.get_message()}"
            )
    

class MessageNewEvent(MessageEvent):
    """新消息事件"""
    detail: MessageDetail
    
    @override
    def get_event_name(self) -> str:
        return "message.new"
    
    @override
    def get_message(self) -> Message:
        return self.message or Message()
    
    @model_validator(mode="after")
    def parse_message_from_detail(self) -> "MessageNewEvent":
        """根据 detail 自动解析并填充 message 字段"""
        if self.detail.content_type == "text/plain":
            self.message = Message(MessageSegment.text(self.detail.content or ""))
        elif self.detail.content_type == "text/markdown":
            self.message = Message(MessageSegment.markdown(self.detail.content or ""))
        elif self.detail.content_type == "vocechat/file":
            file_seg = MessageSegment.file(file_id=self.detail.content or "")
            if self.detail.properties:
                file_seg.data["properties"] = self.detail.properties
            self.message = Message(file_seg)
        else:
            self.message = Message(MessageSegment.text(self.detail.content or ""))
        return self

class NoticeEvent(Event):
    """通知事件"""
    @override
    def get_type(self) -> str:
        return "notice"

class MessageEditEvent(NoticeEvent):
    """消息编辑事件"""
    detail: ReactionDetail

    @override
    def get_event_name(self) -> str:
        return "notice.message.edit"

    @override
    def get_message(self) -> Message:
        return self.get_detail()

    def get_detail(self) -> Message:
        # 编辑后的消息内容
        content = self.detail.detail.get("content", "")  # type: ignore
        
        # 获取消息类型，默认为文本
        content_type = self.detail.detail.get("content_type", "text/plain")  # type: ignore
        
        # 根据消息类型创建消息段
        if content_type == "text/plain":
            return Message(MessageSegment.text(content))
        elif content_type == "text/markdown":
            return Message(MessageSegment.markdown(content))
        elif content_type == "vocechat/file":
            file_seg = MessageSegment.file(file_id= content)
            # 添加文件元数据
            properties = self.detail.detail.get("properties")  # type: ignore
            if properties:
                file_seg.data["properties"] = properties
            return Message(file_seg)
        return Message(MessageSegment.text(content))


class MessageDeleteEvent(NoticeEvent):
    """消息删除事件"""
    detail: ReactionDetail
    
    @override
    def get_event_name(self) -> str:
        return "notice.message.delete"
    
    @property
    def deleted_mid(self) -> int:
        """被删除的消息ID"""
        return self.detail.mid

class FileMessageEvent(MessageNewEvent):
    """文件消息事件"""
    
    @override
    def get_event_name(self) -> str:
        return "message.file"
    
    @property
    def file_path(self) -> str:
        """文件路径"""
        return self.detail.content or ""  # type: ignore
    
    @property
    def file_name(self) -> str:
        """文件名"""
        if self.detail.properties:
            return self.detail.properties.get("name", "")  # type: ignore
        return ""
    
    @property
    def file_size(self) -> int:
        """文件大小"""
        if self.detail.properties:
            return self.detail.properties.get("size", 0)  # type: ignore
        return 0
    
    @property
    def file_type(self) -> str:
        """文件MIME类型"""
        if self.detail.properties:
            return self.detail.properties.get("content_type", "")  # type: ignore
        return ""
    
    @override
    def get_message(self) -> Message:
        # 文件消息特殊处理
        file_seg = MessageSegment.file(file_id= self.detail.content or "")
        # 添加文件元数据
        if self.detail.properties:
            file_seg.data["properties"] = self.detail.properties
        return Message(file_seg)


from typing import Type, Iterable, Union, Dict, List, Any, Optional
from typing_extensions import override

from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment
from pathlib import Path

from .api import ContentType

class File:
    """文件类 方便处理文件"""
    def __init__(
        self,
        path: Optional[Path] = None,
        data: Optional[bytes] = None,
        file_id: Optional[str] = None,
        filename: Optional[str] = None,
    ) -> None:
        if path is None and data is None and file_id is None:
            raise ValueError("Either 'path' 'data' 'file_id' must be provided")
        
        self._path = path
        self._data = data
        self.file_id = file_id
        self.filename = filename

    async def get_data(self) -> bytes:
        if self._data is not None:
            return self._data
        if self._path is not None:
            with open(self._path, "rb") as f:
                return f.read()
        
        raise ValueError("No valid data source provided")

    def __str__(self):
        return self.filename or self.file_id or "Unknown"

class MessageSegment(BaseMessageSegment["Message"]):
    @classmethod
    @override
    def get_message_class(cls) -> Type["Message"]:
        return Message

    @override
    def __str__(self) -> str:
        if self.is_text():
            return self.data.get("text", "")
        elif self.type == "markdown":
            return "[Markdown]"
        elif self.type == "file":
            return f"[File: {self.data.get('file', '')}]"
        return ""
    
    @override
    def is_text(self) -> bool:
        return self.type == "text"
    
    @staticmethod
    def text(text: str) -> "MessageSegment":
        """创建文本消息段"""
        return MessageSegment("text", {"text": text})
    
    @staticmethod
    def markdown(md: str) -> "MessageSegment":
        """创建Markdown消息段"""
        return MessageSegment("markdown", {"text": md})

    @staticmethod
    def file(
        data: Optional[Union[str, bytes, Path]]= None,
        filename: Optional[str] = None,
        file_id: Optional[str] = None
    ) -> "MessageSegment":
        """创建文件消息段"""
        file_obj = None

        if isinstance(data, str):
            data = Path(data)
        
        if isinstance(data, Path):
            file_obj = File(path=data, filename=filename or data.name)
        elif isinstance(data, bytes):
            file_obj = File(data=data, filename=filename)
        elif file_id:
            file_obj = File(file_id= file_id, filename=filename)

        return MessageSegment("file", {"file": file_obj})
    
    @staticmethod
    def mention(user_id: int) -> "MessageSegment":
        """提及消息段"""
        return MessageSegment("text", {"text": f"@{user_id} "})

    def get_content_type(self) -> str:
        """获取消息段对应的内容类型"""
        mapping = {
            "text": ContentType.TEXT_PLAIN,
            "markdown": ContentType.TEXT_MARKDOWN,
            "file": ContentType.VOCECHAT_FILE,
        }
        return mapping.get(self.type, ContentType.TEXT_PLAIN).value
    
    def get_data(self) -> Union[str, Dict[str, Any]]:
        """获取消息段的数据表示"""
        if self.type == "file":
            return {"data": self.data.get("file", "")}
        return self.data.get("text", "")

class Message(BaseMessage[MessageSegment]):
    @classmethod
    @override
    def get_segment_class(cls) -> Type[MessageSegment]:
        return MessageSegment

    @staticmethod
    @override
    def _construct(msg: str) -> Iterable[MessageSegment]:
        # 将字符串构造为文本消息段
        yield MessageSegment.text(msg)
    
    def extract_mentions(self) -> List[int]:
        """从消息中提取提及的用户ID列表"""
        mentions = []
        for segment in self:
            if segment.type == "mention":
                user_id = segment.data.get("user_id")
                if user_id is not None:
                    mentions.append(user_id)
        return mentions
    
    def get_content_type(self) -> str:
        """获取消息的内容类型"""
        # 如果消息包含非文本类型，返回第一个非文本类型的content_type
        for segment in self:
            if segment.type != "text":
                return segment.get_content_type()
        return ContentType.TEXT_PLAIN.value
    
    def get_message_body(self) -> Union[str, Dict[str, Any]]:
        """获取消息的请求体"""
        # 如果消息是纯文本，直接返回字符串
        if all(seg.type == "text" for seg in self):
            return str(self)
        
        # 如果消息包含文件类型，返回文件路径
        for segment in self:
            if segment.type == "file":
                return segment.get_data()
        
        # 否则返回合并后的文本内容
        return str(self)
    
    def __str__(self) -> str:
        """将消息转换为纯文本表示"""
        return "".join(str(seg) for seg in self)
    
    def reduce(self) -> None:
        """合并消息内连续的纯文本段。"""
        index = 1
        while index < len(self):
            if self[index - 1].type == "text" and self[index].type == "text":
                self[index - 1].data["text"] += self[index].data["text"]
                del self[index]
            else:
                index += 1


from typing import Any, Optional, Dict

from nonebot.adapters import Bot as BaseBot

from .event import Event, Target
from .message import Message, MessageSegment, File

class Bot(BaseBot):
    async def call_api(self, api: str, **data) -> Any:
        """调用 OneBot 协议 API。

        参数:
            api: API 名称
            data: API 参数

        返回:
            API 调用返回数据
        """

    async def handle_event(self, event: Event) -> None: ...
    async def send(
        self, event: Event, message: str | Message | MessageSegment, reply: Optional[int], **kwargs: Any
    ) -> Any: ...
    async def download_file(
        self, file_id: str | None = None, message: Message | MessageSegment | None = None
    ) -> bytes: ...
    async def send_message(
        self,
        message: Message,
        *,
        user_id: Optional[int] = None,
        group_id: Optional[int] = None,
        reply: Optional[int] = None,
        **kwargs: Any
    ) -> Any: ...
    async def upload_file(self, file: File) -> Dict[str, Any]: ...
    async def command_add(self, command: str, description: str) -> None: ...
    async def command_get(self) -> None: ...
    async def command_delete(self, id: int) -> None: ...
    async def command_update(self, id: int, command: str, description: str) -> None: ...


from nonebot.utils import logger_wrapper

log = logger_wrapper("vocechat")

def get_mime_type(file_bytes: bytes):
    """通过文件的魔术数字检测文件MIME类型"""
    if len(file_bytes) < 4:
        return 'application/octet-stream'
    
    # 图片类型
    if file_bytes.startswith(b'\xFF\xD8\xFF'):
        return 'image/jpeg'
    elif file_bytes.startswith(b'\x89PNG\r\n\x1a\n'):
        return 'image/png'
    elif file_bytes.startswith(b'GIF87a') or file_bytes.startswith(b'GIF89a'):
        return 'image/gif'
    elif file_bytes.startswith(b'BM'):
        return 'image/bmp'
    elif file_bytes.startswith(b'\x00\x00\x01\x00'):
        return 'image/x-icon'
    elif file_bytes.startswith(b'II*\x00') or file_bytes.startswith(b'MM\x00*'):
        return 'image/tiff'
    elif file_bytes.startswith(b'\x49\x49\x2A\x00') or file_bytes.startswith(b'\x4D\x4D\x00\x2A'):
        return 'image/tiff'
    elif file_bytes.startswith(b'\x0A'):
        return 'image/pcx'
    
    # 文档类型
    elif file_bytes.startswith(b'%PDF'):
        return 'application/pdf'
    elif file_bytes.startswith(b'PK\x03\x04'):  # ZIP格式(也可能是docx, xlsx等)
        # 需要进一步检查是否是Office文档
        if len(file_bytes) > 30 and b'word/' in file_bytes[:1024]:
            return 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
        elif len(file_bytes) > 30 and b'xl/' in file_bytes[:1024]:
            return 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
        elif len(file_bytes) > 30 and b'ppt/' in file_bytes[:1024]:
            return 'application/vnd.openxmlformats-officedocument.presentationml.presentation'
        else:
            return 'application/zip'
    elif file_bytes.startswith(b'\xD0\xCF\x11\xE0\xA1\xB1\x1A\xE1'):
        return 'application/vnd.ms-office'  # 旧的MS Office文档(.doc, .xls等)
    elif file_bytes.startswith(b'{\\rtf'):
        return 'application/rtf'
    
    # 音频/视频类型
    elif file_bytes.startswith(b'RIFF') and len(file_bytes) > 8 and file_bytes[8:12] == b'WAVE':
        return 'audio/wav'
    elif file_bytes.startswith(b'ID3'):
        return 'audio/mpeg'
    elif file_bytes.startswith(b'\xFF\xFB') or file_bytes.startswith(b'\xFF\xF3') or file_bytes.startswith(b'\xFF\xF2'):
        return 'audio/mpeg'  # MP3
    elif file_bytes.startswith(b'OggS'):
        return 'audio/ogg'
    elif file_bytes.startswith(b'fLaC'):
        return 'audio/flac'
    elif file_bytes.startswith(b'RIFF') and len(file_bytes) > 8 and file_bytes[8:12] == b'AVI ':
        return 'video/x-msvideo'
    elif file_bytes.startswith(b'\x00\x00\x00\x20\x66\x74\x79\x70'):
        return 'video/mp4'
    elif file_bytes.startswith(b'\x1A\x45\xDF\xA3'):
        return 'video/webm'  # 也可能是matroska
    
    # 压缩文件
    elif file_bytes.startswith(b'\x1F\x8B\x08'):
        return 'application/gzip'
    elif file_bytes.startswith(b'BZh'):
        return 'application/x-bzip2'
    elif file_bytes.startswith(b'\xFD7zXZ\x00'):
        return 'application/x-xz'
    elif file_bytes.startswith(b'Rar!\x1A\x07\x00') or file_bytes.startswith(b'Rar!\x1A\x07\x01\x00'):
        return 'application/vnd.rar'
    elif file_bytes.startswith(b'7z\xBC\xAF\x27\x1C'):
        return 'application/x-7z-compressed'
    
    # 可执行文件和库
    elif file_bytes.startswith(b'MZ'):
        return 'application/x-msdownload'  # Windows可执行文件
    elif file_bytes.startswith(b'\x7FELF'):
        return 'application/x-executable'  # Linux可执行文件
    
    # 文本和源代码
    elif file_bytes.startswith(b'#!') or file_bytes.startswith(b'\xEF\xBB\xBF') or file_bytes.startswith(b'\xFE\xFF') or file_bytes.startswith(b'\xFF\xFE'):
        return 'text/plain'
    elif b'<?xml' in file_bytes[:100].lower():
        return 'application/xml'
    elif b'<!doctype html' in file_bytes[:100].lower() or b'<html' in file_bytes[:100].lower():
        return 'text/html'
    elif file_bytes.startswith(b'\xFE\xFF') or file_bytes.startswith(b'\xFF\xFE'):
        return 'text/plain'  # Unicode文本
    
    # 其他类型
    elif file_bytes.startswith(b'\x00\x01\x00\x00'):
        return 'application/x-font-ttf'
    elif file_bytes.startswith(b'\x46\x4F\x4E\x54'):
        return 'application/x-font'
    elif file_bytes.startswith(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'):
        return 'application/x-zerosize'  # 可能是空文件
    
    return 'application/octet-stream'  # 默认返回未知二进制流类型


from pydantic import BaseModel, Field
from typing import List

class BotConfig(BaseModel):
    name: str
    user_id: str
    server: str
    api_key: str

class Config(BaseModel):
    vocechat_bots: List[BotConfig] = Field(default_factory=list)